/* ###################################################################
 ** 	Filename	: Events.c
 ** 	Project 	: BP_albrecht_obsazenost_kolejiste_v2_0
 ** 	Processor   : MC9S08DZ96MLF
 ** 	Component   : Events
 ** 	Version 	: Driver 01.02
 ** 	Compiler	: CodeWarrior HCS08 C Compiler
 ** 	Date/Time   : 2019-11-04, 12:52, # CodeGen: 0
 ** 	Abstract	:
 **     	This is user's event module.
 **     	Put your event handler code here.
 ** 	Settings	:
 ** 	Contents	:
 **     	No public methods
 **
 ** ###################################################################*/
/*!
 ** @file Events.c
 ** @version 01.02
 ** @brief
 **     	This is user's event module.
 **     	Put your event handler code here.
 */
/*!
 **  @addtogroup Events_module Events module documentation
 **  @{
 */
/* MODULE Events */

#include "Cpu.h"
#include "Events.h"

/* User includes (#include below this line is not maintained by Processor Expert) */
#include "definition.h"

extern uint8_t CAN_RX_data[13];
extern uint8_t CAN_RX_data_ready;
extern uint8_t TPM1_overflow_cnt;
extern uint8_t TPM2_overflow_cnt;
extern uint8_t LED_RGB_timer;
extern uint8_t LED_RGB_blik;
extern uint8_t CAN_TX_timer;
extern uint16_t meas_current[8];
extern uint8_t LED_RGB_blink;
extern uint8_t rails_timer;
extern uint8_t meas_channel_select;
extern uint8_t meas_current_flag;
extern uint8_t meas_current_total;
extern uint8_t meas_current_total_flag;
extern uint8_t LEDS_occupancy_timer;
extern uint8_t MODE_stisk;
extern uint8_t sending_delay;

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : TPM2_OVF
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(TPM2_OVF) {
	TPM2_overflow_cnt = 1;

	TPM2SC_TOF = 0;
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : AD_conv_complete
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(AD_conv_complete) {
	switch (ADCSC1_ADCH) {   	   		//vbr kanlu AD pevodnku
	//========== PROUD - KANAL .0 ==========
	case measure_current_0: 				//ADCH8
		if ((meas_channel_select & 0x01) == 0x01) {	//je vybrn mc kanl
			meas_current[0] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {									//nen vybrn mc kanl
			meas_current[0] = 0x00;
		}
		ADCSC1 = 0x40 | measure_current_1;	//spust men na kanlu 9
		break;
		//========== PROUD - KANAL .1 ==========
	case measure_current_1:  			 	//ADCH9
		if ((meas_channel_select & 0x02) == 0x02) {
			meas_current[1] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {
			meas_current[1] = 0x00;
		}
		ADCSC1 = 0x40 | measure_current_2;	//spust men na kanlu 10
		break;
		//========== PROUD - KANAL .2 ==========
	case measure_current_2: 			 	//ADCH10
		if ((meas_channel_select & 0x04) == 0x04) {
			meas_current[2] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {
			meas_current[2] = 0x00;
		}
		ADCSC1 = 0x40 | measure_current_3;	//spust men na kanlu 11
		break;
		//========== PROUD - KANAL .3 ==========
	case measure_current_3:  			 	//ADCH11
		if ((meas_channel_select & 0x08) == 0x08) {
			meas_current[3] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {
			meas_current[3] = 0x00;
		}
		ADCSC1 = 0x40 | measure_current_4;	//spust men na kanlu 4
		break;
		//========== PROUD - KANAL .4 ==========
	case measure_current_4: 			 	//ADCH4
		if ((meas_channel_select & 0x10) == 0x10) {
			meas_current[4] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {
			meas_current[4] = 0x00;
		}
		ADCSC1 = 0x40 | measure_current_5;	//spust men na kanlu 5
		break;
		//========== PROUD - KANAL .5 ==========
	case measure_current_5:   			 	//ADCH5
		if ((meas_channel_select & 0x20) == 0x20) {
			meas_current[5] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {
			meas_current[5] = 0x00;
		}
		ADCSC1 = 0x40 | measure_current_6;	//spust men na kanlu 6
		break;
		//========== PROUD - KANAL .6 ==========
	case measure_current_6:  			 	//ADCH6
		if ((meas_channel_select & 0x40) == 0x40) {
			meas_current[6] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {
			meas_current[6] = 0x00;
		}
		ADCSC1 = 0x40 | measure_current_7;	//spust men na kanlu 7
		break;
		//========== PROUD - KANAL .7 ==========
	case measure_current_7:  			 	//ADCH7
		if ((meas_channel_select & 0x80) == 0x80) {
			meas_current[7] = ADCR;    		//uloen zmenen hodnoty do pole
		} else {
			meas_current[7] = 0x00;
		}
		meas_current_flag = 1;				//zmen proud na vech kanlech
		ADCSC1 = 0x1F;   		 			//vypnut men
		break;
	}
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : CAN_RX
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(CAN_RX) {
	uint8_t i;
	for (i = 0; i < 13; i++) {
		CAN_RX_data[0 + i] = *(&CANRIDR0 + i);
	}
	CANRFLG_RXF = 1; 	//clear RX flag
	CAN_RX_data_ready = 1;
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : MODE
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(MODE) {
	//stiskem tlatka MODE se nastavuje perioda odesln namench hodnot 
	MODE_stisk++;
	switch (MODE_stisk) {
	case 1:		//100ms
		sending_delay = 10;
		break;
	case 2:		//1000ms
		sending_delay = 100;
		break;
	case 3:		//2000ms
		sending_delay = 200;
		break;
	case 4:		//vypnuto
		sending_delay = 255;
		MODE_stisk = 0;
		break;
	default:
		sending_delay = 100;
		break;
	}
	PTDSC_PTDACK = 1;
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : TPM1_OVF
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(TPM1_OVF) {
	TPM1_overflow_cnt++;

	//if (ADCSC1 == 0x1F) {   		 //pokud je vypnut AD pevodnk
	ADCSC1 = 0x40 | measure_current_0;   	 //spust men na kanlu 8
	//}
	if (LED_RGB_timer > 0) {		//asova pro blikn RGB LED
		LED_RGB_timer--;
	}
	if (LEDS_occupancy_timer > 0) {	//asova pro blikn LED na kanlech (lut)
		LEDS_occupancy_timer--;
	}
	if (CAN_TX_timer > 0) {			//asova pro periodick odesln po CANu
		CAN_TX_timer--;
	}
	if (rails_timer > 0) {			//asova pro nabit kolej
		rails_timer--;
	}
	TPM1SC_TOF = 0;
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

/* END Events */

/*!
 ** @}
 */
/*
 ** ###################################################################
 **
 ** 	This file was created by Processor Expert 10.3 [05.09]
 ** 	for the Freescale HCS08 series of microcontrollers.
 **
 ** ###################################################################
 */

