/* ###################################################################
 **     Filename    : main.c
 **     Project     : DP_obsazenost_koleji_v2.0
 **     Processor   : MC9S08DZ60MLF
 **     Version     : Driver 01.12
 **     Compiler    : CodeWarrior HCS08 C Compiler
 **     Date/Time   : 2021-11-18, 09:39, # CodeGen: 0
 **     Abstract    :
 **         Main module.
 **         This module contains user's application code.
 **     Settings    :
 **     Contents    :
 **         No public methods
 **
 ** ###################################################################*/
/*!
 ** @file main.c
 ** @version 01.12
 ** @brief
 **         Main module.
 **         This module contains user's application code.
 */
/*!
 **  @addtogroup main_module main module documentation
 **  @{
 */
/* MODULE main */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "PTF.h"
#include "TPM1.h"
#include "PTD.h"
#include "PTD_Int.h"
#include "PTE.h"
#include "MSCAN.h"
#include "PTA.h"
#include "PTB.h"
#include "ADC.h"
#include "WDog1.h"
#include "TPM2.h"
/* Include shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

/* User includes (#include below this line is not maintained by Processor Expert) */

#include "definition.h"

//=========== ID JEDNOTKY ==========
uint32_t CAN_TX_ID_1 = 0x183; //(0x30 0x60) Namen proudy			//0011|0000011-horn 4b=typ(3), doln 7b=slo(3) 
uint32_t CAN_TX_ID_2 = 0x203; //(0x40 0x60) Stavov zprvy			//0100|0000011-horn 4b=typ(4), doln 7b=slo(3) 
uint32_t CAN_RX_ID_1 = 0x283; //(0x50 0x60) Nastaven jednotky		//0101|0000011-horn 4b=typ(5), doln 7b=slo(3)
uint32_t CAN_RX_ID_2 = 0x303; //(0x60 0x60) 						//0110|0000011-horn 4b=typ(6), doln 7b=slo(3)
//==================================
//---------- priklad konfiguracni zpravy pro jednotku ----------
//delka dat  |adresa	|nastav. prodlevy   |prodleva   |crc
//0x02 		  0x50 0x60  0x01 				 0xFA 		 0xFF
//--------------------------------------------------------------
//==================================
uint8_t ID_stand = 0;				//ID standartn
uint8_t ID_exten = 1;				//ID rozen
uint8_t CAN_RX_data[13] = 0;
uint8_t CAN_RX_data_ready = 0;		//petena data z CANu
uint8_t TPM1_overflow_cnt = 0;    	//poet peteen od TPM1
uint8_t TPM2_overflow_cnt = 0;    	//poet peteen od TPM2
uint8_t CAN_TX_timer = 0;
uint8_t i = 0;
uint8_t send_CAN;
uint8_t send_once = 0;
uint8_t H_BRIDGE_SF_last = 1; //uloen poslednho stavu SF H mstku, 1 = ok, 0 = ko
uint8_t MODE_stisk = 2;				//stisknuto tlatko MODE
//								prodleva, zdroj, kanly
uint8_t actual_config[4] = { 0x10, 0x10, 0x00, 0xFF };//pole s aktulnm nastavenm jednotky
uint8_t error[4] = { 0xFF, 0x00, 0x00, 0xFF };		//pole pro odesln chyb
uint8_t send_error = 0;				//posln zprvy o chyb

uint8_t meas_count = 0;				//poet men
uint16_t meas_current[8] = 0;		//pole namench proud
uint16_t avg_current[8] = 0;		//prmr z namench proud
uint16_t send_meas_current[8] = 0;//pole prmrnch namench proud k odesln po CANu
uint8_t meas_current_flag = 0;//zmen proud na vech kanlech - 1 = ano, 0 = ne
uint8_t meas_current_total = 0; 	//hodnota namenho celkovho proudu
uint16_t current_total_max = 1770;	//FB H mstku, maximalni odebirany proud
uint8_t meas_current_total_flag = 0;	//zmen celkov proud
uint8_t sending_delay = 100;		//zpodn odesln po CANu
uint8_t meas_channel_select = 0xFF;	//men na vybranch kanlech
uint8_t rails_timer = 0;
uint8_t rail_train_stop = 3;		//lokomotiva stoj
uint8_t rail_train_go = 70;			//lokomotiva jede
uint8_t detect_current_ch[8] = 0;	//pole pro zaznamenn detekce proudu

uint8_t R = 1;
uint8_t G = 1;
uint8_t B = 1;
uint8_t LED_RGB_blink = 0;
uint8_t LEDS_occupancy_blink = 0;
uint8_t LED_RGB_timer = 0;
uint8_t LEDS_occupancy_timer = 0;
uint8_t LEDS_occup_stop = 0;
uint8_t LEDS_occup_go = 0;

//========== ROZSVICENI LED PRISLUSNYCH KANALU ========== 
void LEDS_start_seq(uint8_t LED_channel, uint8_t on_off) {//rozsvcen LED na kanlech, 0 = vyp., 1 = zap.
	switch (LED_channel) {
	case 0:
		LED_1 = on_off;
		break;
	case 1:
		LED_2 = on_off;
		break;
	case 2:
		LED_3 = on_off;
		break;
	case 3:
		LED_4 = on_off;
		break;
	case 4:
		LED_5 = on_off;
		break;
	case 5:
		LED_6 = on_off;
		break;
	case 6:
		LED_7 = on_off;
		break;
	case 7:
		LED_8 = on_off;
		break;
	}
}
void LEDS_occupancy(uint8_t LED_channel) {//funkce pro ovladani LED diod jednotlivych useku, input je jeden byte, nejnizsi bit je usek 0, nejvyssi bit usek 7
	LED_1 = (LED_channel & 0b00000001);
	LED_2 = (LED_channel & 0b00000010) >> 1;
	LED_3 = (LED_channel & 0b00000100) >> 2;
	LED_4 = (LED_channel & 0b00001000) >> 3;
	LED_5 = (LED_channel & 0b00010000) >> 4;
	LED_6 = (LED_channel & 0b00100000) >> 5;
	LED_7 = (LED_channel & 0b01000000) >> 6;
	LED_8 = (LED_channel & 0b10000000) >> 7;
}
//========== DETEKOVANI PROUDU NA KANALECH ==========
void detect_current() {
	detect_current_ch[0] = detect_current_0;
	detect_current_ch[1] = detect_current_1;
	detect_current_ch[2] = detect_current_2;
	detect_current_ch[3] = detect_current_3;
	detect_current_ch[4] = detect_current_4;
	detect_current_ch[5] = detect_current_5;
	detect_current_ch[6] = detect_current_6;
	detect_current_ch[7] = detect_current_7;
}
//========== ROZSVICENI RGB LED ==========
void RGB_color(uint8_t color) {
	//zhasnut RGB
	LED_RGB_R = 1;                    	//1 = vyp.; 0 = zap.
	LED_RGB_G = 1;
	LED_RGB_B = 1;
	//volba barvy
	switch (color) {
	case RED:
		R = 0;		//1 = vyp.; 0 = zap.
		G = 1;
		B = 1;
		break;
	case GREEN:
		R = 1;
		G = 0;
		B = 1;
		break;
	case BLUE:
		R = 1;
		G = 1;
		B = 0;
		break;
	case YELLOW:
		R = 0;
		G = 0;
		B = 1;
		break;
	case AZURE:
		R = 1;
		G = 0;
		B = 0;
		break;
	case PINK:
		R = 0;
		G = 1;
		B = 0;
		break;
	case WHITE:
		R = 0;
		G = 0;
		B = 0;
		break;
	default:
		break;
	}
}
//========== EFEKT PI SPUTN ==========
void LEDS_start() {	//blikani pri inicializaci
	uint8_t i;
	//bliknut erven
	LED_RGB_R = 0;
	LED_RGB_G = 1;
	LED_RGB_B = 1;
	while (!TPM2_overflow_cnt)
		;
	TPM2_overflow_cnt = 0;
	//postupn rozsvcen LED na kanlech (lut)
	for (i = 0; i < 8; i++) {
		LEDS_start_seq(i, 1);
		while (!TPM2_overflow_cnt)
			;
		TPM2_overflow_cnt = 0;
	}
	//bliknut modr
	LED_RGB_R = 1;
	LED_RGB_G = 1;
	LED_RGB_B = 0;
	while (!TPM2_overflow_cnt)
		;
	TPM2_overflow_cnt = 0;
	//postupn zhasnut LED na kanlech (lut)
	for (i = 0; i < 8; i++) {
		LEDS_start_seq(i, 0);
		while (!TPM2_overflow_cnt)
			;
		TPM2_overflow_cnt = 0;
	}
	//bliknut zelen
	LED_RGB_R = 1;
	LED_RGB_G = 0;
	LED_RGB_B = 1;
	while (!TPM2_overflow_cnt)
		;
	TPM2_overflow_cnt = 0;
}
//========== RESETOVANI H MUSTKU ==========
void H_BRIDGE_reset(void) {
	H_BRIDGE_D1 = 1;
	H_BRIDGE_D2 = 0;
	H_BRIDGE_D1 = 0;
	H_BRIDGE_D2 = 1;
}
//========== OVLADANI ZDROJE 15V/3A ==========
void STEPDOWN_ON_OFF(uint8_t on_off) { //1 = zapnut SD a resetovn H mstku; 0 = vypnut SD
	if (on_off == 1) {
		SD_ON_OFF = 0;		//zapnut SD
		H_BRIDGE_reset();	//resetovn H mstku
	} else {
		H_BRIDGE_D1 = 1;
		SD_ON_OFF = 1;		//vypnut SD
	}
}
//========== ODESILANI PO CANU ==========
//             ID zpravy | standartni/rozsireny | data k odeslani | delka dat           	 
uint8_t CAN_TX(uint32_t CAN_ID, uint8_t ID_Map, uint8_t *data,
		uint8_t data_length) {
	uint8_t i = 0;								//pocet cyklu while
	if (CANTFLG == 0x00) { 						//kontrola zaplneneho bufferu
		return 1;
	}
	if (ID_Map == 0) {							//standartni ramec ID
		CANTIDR0 = (uint8_t) (CAN_ID >> 3);		//nastaveni identifikatoru
		CANTIDR1 = (uint8_t) (CAN_ID << 5);    	//nastaveni identifikatoru
	} else {        							//TODO - rozsireny ramec ID
	}
	CANTDLR = data_length;						//nastaveni delky dat

	while (i < 8) {								//ulozeni dat k odeslani
		if (i >= data_length) {					//kontrola maximalni delky dat
			break;
		}
		*(&CANTDSR0 + i) = *(data + i);			//naplneni registru
		i++;
	}
	CANTBSEL = 0x01; 							//odeslani dat z bufferu 1
	CANTFLG = 0x01; 							//vyprazdneni bufferu

	LED_RGB_blink = 1;							//bliknuti RGB
	RGB_color(GREEN);
	return 0;
}
//****************************************************************************************************
//****************************** ZATEK HLAVNHO PROGRAMU *******************************************
//****************************************************************************************************
void main(void) {
	/* Write your local variable definition here */
	/*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
	PE_low_level_init();
	/*** End of Processor Expert internal initialization.                	***/

	/* Write your code here */
	/* For example: for(;;) { } */

	LEDS_start();			//probliknut vemi LED pi startu
	STEPDOWN_ON_OFF(1);		//zapnut stepdownu
	//H_BRIDGE_reset();

	//============================== ZATEK NEKONEN SMYKY ==================================================
	while (1) {
		SRS = 0x55;		//resetovani WatchDogTimeru pro hard reset 
		SRS = 0xAA;
		//====================================================================================================
		//========== PRICHOZI ZPRAVA PO CANU ==========
		if (CAN_RX_data_ready == 1) {					//pili data po CANu?
			CAN_RX_data_ready = 0;			//nulovn pro dal pijet zprvy
			LED_RGB_blink = 1;		//bliknut RGB
			RGB_color(BLUE);		//modr barva
			switch (CAN_RX_data[4]) {			//prvn bajt dat pchoz zprvy
			//---------- HARD RESET JEDNOTKY ----------
			case 0x01:
				if (CAN_RX_data[5] == 0x01) {
					SRS = 0x00;							//hard reset jednotky
					if ((error[2] & 0x80) == 0x80) {	//nastala pedtm chyba?
						error[2] = error[2] ^ 0x80;		//vynulovn chyby
					}
				} else {
					error[2] = error[2] | 0x80;			//chyba
					send_error = 1;						//povolen zasln chyby
				}
				break;
				//---------- NASTAVEN PRODLEVY ODESLAN ----------
			case 0x02:
				if ((CAN_RX_data[5] >= 0x01) && (CAN_RX_data[5] <= 0xFA)) {	//perioda odesln 10 a 2500 ms
					sending_delay = CAN_RX_data[5];	//druh bajt dat uruje zpodn
				} else if (CAN_RX_data[5] == 0x00) {
					send_once = 1;					//odesln jednou
				} else if (CAN_RX_data[5] == 0xFF) {
					sending_delay = 0xFF;			//odesln vypnuto
				} else {
					error[2] = error[2] | 0x40;		//chyba
					send_error = 1;					//povolen zasln chyby
				}
				if ((error[2] & 0x40) == 0x40) {	//nastala pedtm chyba?
					error[2] = error[2] ^ 0x40;		//vynulovn chyby
				}
				CAN_TX_timer = sending_delay;//piazen zpodn pro odesln
				actual_config[1] = sending_delay;//uloen do aktulnho stavu jednotky
				break;
				//---------- RESETOVAN H MSTKU ----------
			case 0x03:
				if (CAN_RX_data[5] == 0x01) {
					H_BRIDGE_reset();					//reset H mstku
					if ((actual_config[2] & 0x80) == 0x80) {
						actual_config[2] = actual_config[2] ^ 0x80;	//uloen do aktulnho stavu jednotky
					}
					if ((error[2] & 0x20) == 0x20) {	//nastala pedtm chyba?
						error[2] = error[2] ^ 0x20; 	//vynulovn chyby
					}
					if ((error[1] & 0x80) == 0x80) { 	//pokud byl peten
						error[1] = error[1] ^ 0x80; 	//vynulovn chyby 
					}
				} else {
					error[2] = error[2] | 0x20;			//chyba pi resetu
					send_error = 1;						//povolen zasln chyby
				}
				break;
				//---------- NASTAVEN ZDROJE 15V/3A ----------
			case 0x04:
				//stepdown off
				if (CAN_RX_data[5] == 0x00) {
					STEPDOWN_ON_OFF(0);				//funkce pro vypnut zdroje
					actual_config[2] = actual_config[2] ^ 0x40;	//uloen do aktulnho stavu jednotky
					error[1] = error[1] | 0x40;			//chyba - vypnut zdroj
					send_error = 1;						//povolen zasln chyby
					break;
				}
				//stepdown on
				else if (CAN_RX_data[5] == 0x01) {
					STEPDOWN_ON_OFF(1);				//funkce pro zapnut zdroje
					actual_config[2] = actual_config[2] | 0x40;	//uloen do aktulnho stavu jednotky
					if ((error[1] & 0x40) == 0x40) {//pokud byla chyba vypnutho zdroje
						error[1] = error[1] ^ 0x40;		//vynulovn chyby 
					}
					break;
				} else {
					error[2] = error[2] | 0x10;	//chyba pi nastavovn zdroje
					send_error = 1;						//povolen zasln chyby
					break;
				}
				break;
				//---------- MREN NA VYBRANCH KANALECH ----------
			case 0x05:
				meas_channel_select = CAN_RX_data[5];//peten nastaven kanlu z pchoz zprvy
				actual_config[3] = meas_channel_select;	//uloen do aktulnho stavu jednotky
				break;
				//---------- PRETEN AKTUALNHO NASTAVEN JEDNOTKY ----------
			case 0x10:
				send_CAN = CAN_TX(CAN_TX_ID_2, ID_stand, &actual_config, 4);
				break;
				//---------- NASTAVEN ID SLA JEDNOTKY ----------
			case 0x20:
				//TODO
				break;
				//--------------------------------------------------
			default:
				break;
			}
		}
		//====================================================================================================
		//========= DETEKCE PROUDU NA KANALECH ==========
		if (rails_timer == 0) {							//koleje jsou nabite
			if (meas_current_total_flag == 1) {
				//celkovy proud presahl 3A -> H-mustek do vysoke impedance
				if (meas_current_total > current_total_max) {
					H_BRIDGE_D1 = 1;
					actual_config[2] = actual_config[2] | 0x80;	//uloen do aktulnho stavu jednotky
					error[1] = error[1] | 0x80;		//chyba o peten H-mstku
				}
				meas_current_total_flag = 0;
			} else {
				ADCSC1 = 0x40 | measure_total_current; //zmen celkovho proudu od H-mstku
				meas_current_total_flag = 1;			//zmen celkov proud
			}
		}
		//spusti se kdyz poet peteen TPM1 je vt ne zpodn men na kanlech, je zapnut odesln a jednotka nen v chyb
		if ((TPM1_overflow_cnt >= sending_delay) && (sending_delay)
				&& (!send_error)) {
			ADCSC1 = 0x40 | measure_current_0; //sputen prvnho AD pevodnku, spust i vechny ostatn
			TPM1_overflow_cnt = 0;					//vynulovn peteen TPM1	
		}

		if (meas_current_flag == 1) {		//zmen odbr na vech kanlech?
			for (i = 0; i < 8; i++) {				//uloen namench hodnot
				avg_current[i] += meas_current[i];
			}
			meas_count++;								//poet men

			detect_current();				//funkce pro detekci odbru proudu
			if (meas_count >= 8) {						//provedlo se 8 men?
				LEDS_occup_go = 0;		//nulovn promnnch pro ovldn LED
				LEDS_occup_stop = 0;
				for (i = 0; i < 8; i++) {
					avg_current[i] /= 8;		//zprmrovn namench hodnot
					send_meas_current[i] = avg_current[i];	//uloen hodnot
					//je detekovn proud a lokomotiva stoj?
					if ((send_meas_current[i] >= rail_train_stop)
							&& (detect_current_ch[i] == 1)) {
						LEDS_occup_stop = LEDS_occup_stop | (1 << i);//povolen rozsvcen LED kanl (stojc vlak)
					}
					//je detekovn proud a lokomotiva jede?
					if ((send_meas_current[i] >= rail_train_go)
							&& (detect_current_ch[i] == 1)) {
						LEDS_occup_go = LEDS_occup_go | (1 << i);//povolen rozsvcen LED kanl (jedouc vlak)
					}
					avg_current[i] = 0;		//vynulovn zprmrovanch hodnot
				}
				meas_count = 0;						//vynulovn potu men
			}
			meas_current_flag = 0;				//vynulovn pro dal men
		}
		//====================================================================================================
		//========== PERIODICKE ODESILANI PO CANU ==========
		if (CAN_TX_timer == 0 && sending_delay != 0 && error[1] == 0x00 && error[2] == 0x00) {
			CAN_TX_timer = sending_delay;
			if (sending_delay != 0xFF && send_once != 1) {//pokud nen vypnut odesln nebo zapnut odesln pouze jednou
				send_CAN = CAN_TX(CAN_TX_ID_1, ID_stand, &send_meas_current, 8);//odesln namench hodnot proudu na kanlech
			} else {	//vypnut odesln

			}
		}
		if (send_once == 1) {	//odele pouze jednou
			send_CAN = CAN_TX(CAN_TX_ID_1, ID_stand, &send_meas_current, 8);//odesln namench hodnot proudu na kanlech
			send_once = 0;
		}
		//====================================================================================================
		//========== PRETIZENI H MUSTKU ==========
		if (H_BRIDGE_SF == 0) {					//peten H mstku za provozu
			error[1] = error[1] | 0x80;					//chyba o peten
			if (H_BRIDGE_SF != H_BRIDGE_SF_last) {//pokud se zmnil stav H-mstku
				send_error = 1;							//povolen zasln chyby
				STEPDOWN_ON_OFF(0);					//funkce pro vypnut zdroje
				STEPDOWN_ON_OFF(1);					//funkce pro zapnut zdroje
			} else {
			}
			H_BRIDGE_SF_last = H_BRIDGE_SF;	//uloen poslednho stavu H-mstku
		}
		//====================================================================================================
		//========== SIGNALIZACE A HLASENI CHYBY ==========
		if ((error[1] != 0x00) || (error[2] != 0x00)) {	//pokud nkde nastala chyba
			if ((error[1] & 0x80) == 0x80) {		//chyba peten H-mstku
				RGB_color(RED);							//erven barva
			}
			if ((error[1] & 0x40) == 0x40) { //pokud je vypnut zdroj 15V/3A
				RGB_color(YELLOW);						//lut barva
			}
			if ((error[1] & 0x20) == 0x20) { //chyba pi odesln namench hodnot 
				RGB_color(AZURE);						//tyrkysov barva
			}
			LED_RGB_R = R;
			LED_RGB_G = G;
			LED_RGB_B = B;
			if (send_error == 1) {				//pokud m poslat zprvu o chyb
				send_CAN = CAN_TX(CAN_TX_ID_2, ID_stand, &error, 4);	//hlen chyby
				send_error = 0;					//vynulovn pro dal odesln
			}
		}
		//====================================================================================================
		//========== BLIKANI LED ==========
		if (LED_RGB_blink > 0) {						//bliknut LED?
			if (sending_delay >= 50) {//pokud perioda odesln je vt jak 50
				LED_RGB_timer = 20;						//blikej periodou 20
			} else {									//pokud je men jak 50
				LED_RGB_timer = 2;						//blikej rychleji
			}
			LED_RGB_R = R;
			LED_RGB_G = G;
			LED_RGB_B = B;
			LED_RGB_blink = 0;
		}
		if (LED_RGB_timer == 0) {	//zhasnut RGB
			LED_RGB_R = 1;
			LED_RGB_G = 1;
			LED_RGB_B = 1;
		}
		if ((error[1] == 0x00) && (error[2] == 0x00)) {		//bez chybov chod jednotky?
			if (TPM1_overflow_cnt >= 5) {				//pokud ubhlo 50 ms
				if (LEDS_occupancy_blink) {				//blikn LED?
					LEDS_occupancy(LEDS_occup_stop);//rozsvcen LED pro stojc vlak
				} else {
					//XOR - pomoc nho blik LED kanlu kde jede vlak
					LEDS_occupancy(LEDS_occup_stop ^ LEDS_occup_go); //rozsvcen LED pro jedouc vlak
				}
				LEDS_occupancy_blink = !LEDS_occupancy_blink;	//blikn LED
				TPM1_overflow_cnt = 0;				//vynulovn peteen TPM1
			}
		}
		//====================================================================================================
	}
	//============================== KONEC NEKONEN SMYKY ==================================================

	//****************************************************************************************************
	//****************************** KONEC HLAVNIHO PROGRAMU *********************************************
	//****************************************************************************************************
	/*** Don't write any code pass this line, or it will be deleted during code generation. ***/
	/*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
#ifdef PEX_RTOS_START
	PEX_RTOS_START(); /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
#endif
	/*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;){}
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
 ** @}
 */
/*
 ** ###################################################################
 **
 **     This file was created by Processor Expert 10.3 [05.09]
 **     for the Freescale HCS08 series of microcontrollers.
 **
 ** ###################################################################
 */
