/* ###################################################################
**     This component module is generated by Processor Expert. Do not modify it.
**     Filename    : MSCAN.c
**     Project     : DP_prestavniky_v1.1
**     Processor   : MC9S08DZ128MLF
**     Component   : Init_MSCAN
**     Version     : Component 01.004, Driver 01.07, CPU db: 3.00.038
**     Compiler    : CodeWarrior HCS08 C Compiler
**     Date/Time   : 2022-04-11, 20:36, # CodeGen: 0
**     Abstract    :
**          This file implements the MSCAN (MSCAN) module initialization
**          according to the Peripheral Initialization Component settings,
**          and defines interrupt service routines prototypes.
**          The Motorola Scalable Controller Area Network (MSCAN) definition
**          is based on is the specific
**          implementation of the Motorola Scalable CAN concept targeted
**          for the Freescale HCS08 Microcontroller Family.
**
**          The basic features of the MSCAN are as follows:
**          - Implementation of the CAN protocol - Version 2.0A/B
**           Standard and extended data frames
**           0 - 8 bytes data length
**           Programmable bit rate up to 1 Mbps1
**           Support for remote frames
**           5 receive buffers with FIFO storage scheme
**          - 3 transmit buffers with internal prioritization using a local
**          priority concept:
**          - Flexible maskable identifier filter supports two full size
**          extended identifier filters (two 32-bit) or four 16-bit filters
**          or eight 8-bit filters
**          - Programmable wake-up functionality with integrated low-pass
**          filter
**          - Programmable loop back mode supports self-test operation
**          - Programmable listen-only mode for monitoring of CAN bus
**          - Separate signalling and interrupt capabilities for all CAN
**          receiver and transmitter error states
**          (Warning, Error Passive, Bus-Off)
**          - Programmable MSCAN clock source either Bus Clock or Oscillator
**          Clock
**          - Internal timer for time-stamping of received and transmitted
**          messages
**          - Three low power modes: Sleep, Power Down and MSCAN Enable
**          - Global initialization of configuration registers
**     Settings    :
**          Component name                                 : MSCAN
**          Device                                         : MSCAN
**          Clock Source                                   : Bus Clock
**          Baud Rate Prescaler                            : 2
**          Synchr. Jump Width                             : 1
**          Sampling                                       : One sample per bit
**          Time Segment 1                                 : 7
**          Time Segment 2                                 : 8
**          CAN frequency                                  : 8 MHz
**          Time quantum                                   : 125 ns
**          Bit rate                                       : 500 kbit/s
**          CAN Stops in Wait Mode                         : no
**          Wake-Up Mode                                   : None
**          Loop Back Test Mode                            : Disabled
**          Listen Only Mode                               : Normal operation
**          Sleep Mode Request                             : Disabled
**          Time Stamp                                     : Disabled
**          Acceptance mode                                : Four 16 bit Acceptance Filters
**          Rx acceptance ID(1st bank)                     : 70207020
**          Rx acceptance ID(2nd bank)                     : 0
**          Rx acceptance ID mask (1st bank)               : 0
**          Rx acceptance ID mask (2nd bank)               : 0
**          RXCAN pin                                      : PTE7_RxD2_RxCAN
**          RXCAN pin signal                               : 
**          TXCAN pin                                      : PTE6_TxD2_TxCAN
**          TXCAN pin signal                               : 
**          Wake up                                        : 
**          Wake up                                        : Disabled
**          Interrupt                                      : Vcanwu
**          ISR name                                       : 
**          Error                                          : 
**          Error Interrupt                                : Vcanerr
**          Status Change Interrupt                        : Disabled
**          Receiver Status Change                         : do not generate
**          Transmitt. Status Change                       : do not generate
**          Overrun Interrupt                              : Disabled
**          ISR name                                       : 
**          Receiver Full                                  : 
**          Receiver Full                                  : Enabled
**          Receiver Interrupt                             : Vcanrx
**          ISR name                                       : CAN_RX
**          Transmitter empty                              : 
**          Transmitter Interrupt                          : Vcantx
**          Tx Empty Interrupt 0                           : Disabled
**          Tx Empty Interrupt 1                           : Disabled
**          Tx Empty Interrupt 2                           : Disabled
**          ISR name                                       : 
**          Call Init in CPU init. code                    : yes
**          CAN Enable                                     : yes
**     Contents    :
**         Init - void MSCAN_Init(void);
**
**     Copyright : 1997 - 2014 Freescale Semiconductor, Inc. 
**     All Rights Reserved.
**     
**     Redistribution and use in source and binary forms, with or without modification,
**     are permitted provided that the following conditions are met:
**     
**     o Redistributions of source code must retain the above copyright notice, this list
**       of conditions and the following disclaimer.
**     
**     o Redistributions in binary form must reproduce the above copyright notice, this
**       list of conditions and the following disclaimer in the documentation and/or
**       other materials provided with the distribution.
**     
**     o Neither the name of Freescale Semiconductor, Inc. nor the names of its
**       contributors may be used to endorse or promote products derived from this
**       software without specific prior written permission.
**     
**     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
**     ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
**     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
**     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
**     ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
**     (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
**     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
**     ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
**     (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
**     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**     
**     http: www.freescale.com
**     mail: support@freescale.com
** ###################################################################*/
/*!
** @file MSCAN.c
** @version 01.07
** @brief
**          This file implements the MSCAN (MSCAN) module initialization
**          according to the Peripheral Initialization Component settings,
**          and defines interrupt service routines prototypes.
**          The Motorola Scalable Controller Area Network (MSCAN) definition
**          is based on is the specific
**          implementation of the Motorola Scalable CAN concept targeted
**          for the Freescale HCS08 Microcontroller Family.
**
**          The basic features of the MSCAN are as follows:
**          - Implementation of the CAN protocol - Version 2.0A/B
**           Standard and extended data frames
**           0 - 8 bytes data length
**           Programmable bit rate up to 1 Mbps1
**           Support for remote frames
**           5 receive buffers with FIFO storage scheme
**          - 3 transmit buffers with internal prioritization using a local
**          priority concept:
**          - Flexible maskable identifier filter supports two full size
**          extended identifier filters (two 32-bit) or four 16-bit filters
**          or eight 8-bit filters
**          - Programmable wake-up functionality with integrated low-pass
**          filter
**          - Programmable loop back mode supports self-test operation
**          - Programmable listen-only mode for monitoring of CAN bus
**          - Separate signalling and interrupt capabilities for all CAN
**          receiver and transmitter error states
**          (Warning, Error Passive, Bus-Off)
**          - Programmable MSCAN clock source either Bus Clock or Oscillator
**          Clock
**          - Internal timer for time-stamping of received and transmitted
**          messages
**          - Three low power modes: Sleep, Power Down and MSCAN Enable
**          - Global initialization of configuration registers
*/         
/*!
**  @addtogroup MSCAN_module MSCAN module documentation
**  @{
*/         
/* MODULE MSCAN. */
#include "MSCAN.h"

/*
** ###################################################################
**
**  The interrupt service routine(s) must be implemented
**  by user in one of the following user modules.
**
**  If the "Generate ISR" option is enabled, Processor Expert generates
**  ISR templates in the CPU event module.
**
**  User modules:
**      main.c
**      Events.c
**
** ###################################################################
ISR(CAN_RX)
        {
        // NOTE: The routine should include the following actions to obtain
        //       correct functionality of the hardware.
        //
        //      The ISR is invoked by RXF flag. The RXF flag is cleared
        //      if a "1" is written to the flag in CANRFLG register.
        //      Example: CANRFLG = CANRFLG_RXF_MASK;
        }
*/
/*
** ===================================================================
**     Method      :  MSCAN_Init (component Init_MSCAN)
**     Description :
**         This method initializes registers of the CAN module
**         according to this Peripheral Initialization Component settings.
**         Call this method in user code to initialize the module.
**         By default, the method is called by PE automatically; see
**         "Call Init method" property of the component for more details.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void MSCAN_Init(void)
{
  /* CANCTL0: INITRQ=1 */
  setReg8Bits(CANCTL0, 0x01U);          
  while(CANCTL1_INITAK == 0U) {        /* Wait for init acknowledge */
   SRS = 0x55U;                        /* Reset watchdog counter write 55, AA */
   SRS = 0xAAU;
  }
  /* CANCTL1: CANE=1,CLKSRC=1,LOOPB=0,LISTEN=0,BORM=0,WUPM=0,SLPAK=0,INITAK=1 */
  setReg8(CANCTL1, 0xC1U);              
  /* CANBTR1: SAMP=0,TSEG22=1,TSEG21=1,TSEG20=1,TSEG13=0,TSEG12=1,TSEG11=1,TSEG10=0 */
  setReg8(CANBTR1, 0x76U);              
  /* CANBTR0: SJW1=0,SJW0=0,BRP5=0,BRP4=0,BRP3=0,BRP2=0,BRP1=0,BRP0=1 */
  setReg8(CANBTR0, 0x01U);              
  /* CANIDAC: ??=0,??=0,IDAM1=0,IDAM0=1,??=0,IDHIT2=0,IDHIT1=0,IDHIT0=0 */
  setReg8(CANIDAC, 0x10U);              
  /* CANIDAR0: AC7=0,AC6=1,AC5=1,AC4=1,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR0, 0x70U);             
  /* CANIDAR1: AC7=0,AC6=0,AC5=1,AC4=0,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR1, 0x20U);             
  /* CANIDAR2: AC7=0,AC6=1,AC5=1,AC4=1,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR2, 0x70U);             
  /* CANIDAR3: AC7=0,AC6=0,AC5=1,AC4=0,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR3, 0x20U);             
  /* CANIDAR4: AC7=0,AC6=0,AC5=0,AC4=0,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR4, 0x00U);             
  /* CANIDAR5: AC7=0,AC6=0,AC5=0,AC4=0,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR5, 0x00U);             
  /* CANIDAR6: AC7=0,AC6=0,AC5=0,AC4=0,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR6, 0x00U);             
  /* CANIDAR7: AC7=0,AC6=0,AC5=0,AC4=0,AC3=0,AC2=0,AC1=0,AC0=0 */
  setReg8(CANIDAR7, 0x00U);             
  /* CANIDMR0: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR0, 0x00U);             
  /* CANIDMR1: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR1, 0x00U);             
  /* CANIDMR2: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR2, 0x00U);             
  /* CANIDMR3: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR3, 0x00U);             
  /* CANIDMR4: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR4, 0x00U);             
  /* CANIDMR5: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR5, 0x00U);             
  /* CANIDMR6: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR6, 0x00U);             
  /* CANIDMR7: AM7=0,AM6=0,AM5=0,AM4=0,AM3=0,AM2=0,AM1=0,AM0=0 */
  setReg8(CANIDMR7, 0x00U);             
  /* CANCTL0: INITRQ=0 */
  clrReg8Bits(CANCTL0, 0x01U);          
  while(CANCTL1_INITAK != 0U) {        /* Wait for init exit */
   SRS = 0x55U;                        /* Reset watchdog counter write 55, AA */
   SRS = 0xAAU;
  }
  /* CANCTL0: RXFRM=0,RXACT=0,CSWAI=0,SYNCH=0,TIME=0,WUPE=0,SLPRQ=0,INITRQ=0 */
  setReg8(CANCTL0, 0x00U);              
  /* CANRIER: WUPIE=0,CSCIE=0,RSTATE1=0,RSTATE0=0,TSTATE1=0,TSTATE0=0,OVRIE=0,RXFIE=1 */
  setReg8(CANRIER, 0x01U);              
  /* CANTIER: ??=0,??=0,??=0,??=0,??=0,TXEIE2=0,TXEIE1=0,TXEIE0=0 */
  setReg8(CANTIER, 0x00U);              
}

/* END MSCAN. */

/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.3 [05.09]
**     for the Freescale HCS08 series of microcontrollers.
**
** ###################################################################
*/
