/* ###################################################################
 **     Filename    : Events.c
 **     Project     : Zkousime
 **     Processor   : MC9S08DZ60MLF
 **     Component   : Events
 **     Version     : Driver 01.02
 **     Compiler    : CodeWarrior HCS08 C Compiler
 **     Date/Time   : 2019-11-13, 10:24, # CodeGen: 0
 **     Abstract    :
 **         This is user's event module.
 **         Put your event handler code here.
 **     Settings    :
 **     Contents    :
 **         No public methods
 **
 ** ###################################################################*/
/*!
 ** @file Events.c
 ** @version 01.02
 ** @brief
 **         This is user's event module.
 **         Put your event handler code here.
 */
/*!
 **  @addtogroup Events_module Events module documentation
 **  @{
 */
/* MODULE Events */

#include "Cpu.h"
#include "Events.h"

/* User includes (#include below this line is not maintained by Processor Expert) */
#include "Definice.h"

extern uint8_t RX_message[13];
extern uint8_t RX_data_ready;
extern uint8_t prodleva;
extern uint8_t TPM1_overflow_cnt;    	//poet peteen od TPM1
extern uint8_t TPM2_overflow_cnt;    	//poet peteen od TPM2
extern uint8_t TPM3_overflow_cnt;    	//poet peteen od TPM3
extern uint8_t CAN_TX_timer;
extern uint8_t LED_RGB_timer;
extern uint8_t LED_RGB_blink;

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : TPM1_preteceni
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(TPM1_preteceni) {

}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : CAN_RX
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(CAN_RX) {
	uint8_t i;
	for (i = 0; i < 13; i++) {
		RX_message[0 + i] = *(&CANRIDR0 + i);
	}
	CANRFLG_RXF = 1; 	//clear RX flag
	RX_data_ready = 1;
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : TPM2_OVF
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(TPM2_OVF) {
	//TPM2_overflow_cnt = 1;

	//TPM2SC_TOF = 0;
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG __NEAR_SEG NON_BANKED
#endif
/*
 ** ===================================================================
 **     Interrupt handler : TPM3_OVF
 **
 **     Description :
 **         User interrupt service routine. 
 **     Parameters  : None
 **     Returns     : Nothing
 ** ===================================================================
 */ISR(TPM3_OVF) {
	TPM3_overflow_cnt++;

	if (CAN_TX_timer > 0) {			//asova pro periodick odesln po CANu
		CAN_TX_timer--;
	}
	if (LED_RGB_timer > 0) {		//asova pro blikn RGB LED
		LED_RGB_timer--;
	}
	if (prodleva > 0) {				//asova pro zpodn mezi otenm jednotlivch serv
		prodleva--;
	}
	TPM3SC_TOF = 0;
}
#ifdef ISR_IN_NONBANKED
#pragma CODE_SEG DEFAULT
#endif

/* END Events */

/*!
 ** @}
 */
/*
 ** ###################################################################
 **
 **     This file was created by Processor Expert 10.3 [05.09]
 **     for the Freescale HCS08 series of microcontrollers.
 **
 ** ###################################################################
 */
