/* ###################################################################
 **     Filename    : main.c
 **     Project     : vlaky
 **     Processor   : MC9S08DZ60MLF
 **     Version     : Driver 01.12
 **     Compiler    : CodeWarrior HCS08 C Compiler
 **     Date/Time   : 2020-03-17, 14:48, # CodeGen: 0
 **     Abstract    :
 **         Main module.
 **         This module contains user's application code.
 **     Settings    :
 **     Contents    :
 **         No public methods
 **
 ** ###################################################################*/
/*!
 ** @file main.c
 ** @version 01.12
 ** @brief
 **         Main module.
 **         This module contains user's application code.
 */
/*!
 **  @addtogroup main_module main module documentation
 **  @{
 */
/* MODULE main */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "PTA.h"
#include "TPM2.h"
#include "MSCAN.h"
#include "TPM3.h"
#include "WDog1.h"
#include "TPM1.h"
#include "PTB.h"
#include "PTE.h"
/* Include shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

/* User includes (#include below this line is not maintained by Processor Expert) */
#include "Definice.h"
//====================== ID JEDNOTKY =====================
uint32_t CAN_TX_ID_1 = 0x401; //(0x80 0x20) Pozice serv
uint32_t CAN_RX_ID_1 = 0x381; //(0x70 0x20) Nastaven jednotky
//========================================================
//---------- priklad konfiguracni zpravy pro jednotku ----------
//delka dat  |adresa	|nastav. prodlevy   |prodleva   |crc
//0x02 		  0x70 0x20  0x01 				 0xFA 		 0xFF
//--------------------------------------------------------------
uint8_t ID_stand = 0;			//ID standartn
uint8_t ID_exten = 1;			//ID rozen
uint8_t RX_message[13] = 0;
uint8_t RX_data_ready = 0;
uint16_t Dorazy = 0;
uint8_t i, j = 0;
uint8_t prodleva = 0;
uint8_t S_move_delay = 20;		//zpodn mezi pohybem jednotlivch serv
uint8_t S_move_inc = 0;			//pomocn promnn pro vbr serva
uint8_t S_move_flag = 0; 		//flag pro proveden natoen serv
uint8_t S_power[8] = { 1, 1, 1, 1, 1, 1, 1, 1 };		//zap/vyp serv
//natoen serv - 260 = 0, 335 = 15, 635 = 75, 760 = 90, 785 = 105, 810 = 110, 1085 = 165, 1260 = 180
uint16_t S_pos[8] = { 785, 785, 785, 785, 785, 785, 785, 785 };	//pozice serv
uint16_t S_stopL[8] = { 760, 760, 760, 760, 760, 760, 760, 760 };//dorazy vlevo
uint16_t S_stopR[8] = { 810, 810, 810, 810, 810, 810, 810, 810 }; //dorazy vpravo
uint8_t S_resp[8] = { 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69, 0x69 };	//zptn vazba o pozici serv ve stupnch (105)
uint8_t S_resp_LR[8] = { 0xF2, 0xF2, 0xF2, 0xF2, 0xF2, 0xF2, 0xF2, 0xF2 };//zptn vazba o pozici serv vlevo/vpravo/mezi
uint8_t send_CAN;
uint8_t CAN_TX_timer = 0;
uint8_t sending_delay = 100;	//zpodn odesln po CANu
uint8_t send_once = 0;			//pro odesln na popud
uint8_t send_LR = 0;			//pro odesln pozic serv vlevo/vpravo
uint8_t TPM1_overflow_cnt = 0;  //poet peteen od TPM1
uint8_t TPM2_overflow_cnt = 0;  //poet peteen od TPM2
uint8_t TPM3_overflow_cnt = 0;  //poet peteen od TPM3
uint16_t send_position[8] = 0;	//pole pozic servo motor k odesln po CANu
uint8_t error[4] = { 0xFF, 0x00, 0xFF };		//pole pro odesln chyb
uint8_t send_error = 0;			//posln zprvy o chyb

uint8_t R = 1;
uint8_t G = 1;
uint8_t B = 1;
uint8_t LED_RGB_blink = 0;
uint8_t LED_RGB_timer = 0;

//========== ROZSVICENI RGB LED ==========
void RGB_color(uint8_t color) {
	//zhasnut RGB
	LED_RGB_R = 1;                    	//1 = vyp.; 0 = zap.
	LED_RGB_G = 1;
	LED_RGB_B = 1;
	//volba barvy
	switch (color) {
	case RED:
		R = 0;		//1 = vyp.; 0 = zap.
		G = 1;
		B = 1;
		break;
	case GREEN:
		R = 1;
		G = 0;
		B = 1;
		break;
	case BLUE:
		R = 1;
		G = 1;
		B = 0;
		break;
	case YELLOW:
		R = 0;
		G = 0;
		B = 1;
		break;
	case AZURE:
		R = 1;
		G = 0;
		B = 0;
		break;
	case PINK:
		R = 0;
		G = 1;
		B = 0;
		break;
	case WHITE:
		R = 0;
		G = 0;
		B = 0;
		break;
	default:
		break;
	}
}
//========== ODESILANI PO CANU ==========
//             ID zpravy | standart/rozsireny | data k odeslani | delka dat           	 
uint8_t CAN_TX(uint32_t CAN_ID, uint8_t ID_Map, uint8_t *data,
		uint8_t data_length) {
	uint8_t i = 0;								//pocet cyklu while
	if (CANTFLG == 0x00) { 						//kontrola zaplneneho bufferu
		return 1;
	}
	if (ID_Map == 0) {							//standartni ramec ID
		CANTIDR0 = (uint8_t) (CAN_ID >> 3);		//nastaveni identifikatoru
		CANTIDR1 = (uint8_t) (CAN_ID << 5);    	//nastaveni identifikatoru
	} else {        							//TODO - rozsireny ramec ID
	}
	CANTDLR = data_length;						//nastaveni delky dat

	while (i < 8) {								//ulozeni dat k odeslani
		if (i >= data_length) {					//kontrola maximalni delky dat
			break;
		}
		*(&CANTDSR0 + i) = *(data + i);			//naplneni registru
		i++;
	}
	CANTBSEL = 0x01; 							//odeslani dat z bufferu 1
	CANTFLG = 0x01; 							//vyprazdneni bufferu

	LED_RGB_blink = 1;							//bliknuti RGB
	RGB_color(GREEN);
	return 0;
}
//========== ZAPNUTI/VYPNUTI SERV ==========
void S_ON_OFF(uint8_t servo, uint8_t on_off) {	//0=zap, 1=vyp
	switch (servo) {
	case 1:	//servo1
		S1_ONOFF = on_off;			//zap/vyp serva
		S_power[0] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 2:	//servo2
		S2_ONOFF = on_off;			//zap/vyp serva
		S_power[1] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 3:	//servo3
		S3_ONOFF = on_off;			//zap/vyp serva
		S_power[2] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 4:	//servo4
		S4_ONOFF = on_off;			//zap/vyp serva
		S_power[3] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 5:	//servo5
		S5_ONOFF = on_off;			//zap/vyp serva
		S_power[4] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 6:	//servo6
		S6_ONOFF = on_off;			//zap/vyp serva
		S_power[5] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 7:	//servo7
		S7_ONOFF = on_off;			//zap/vyp serva
		S_power[6] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 8:	//servo8
		S8_ONOFF = on_off;			//zap/vyp serva
		S_power[7] = on_off;		//uloen stavu o zap/vyp serva
		break;
	case 10: //zap/vyp vech serv
		S1_ONOFF = on_off;
		S2_ONOFF = on_off;
		S3_ONOFF = on_off;
		S4_ONOFF = on_off;
		S5_ONOFF = on_off;
		S6_ONOFF = on_off;
		S7_ONOFF = on_off;
		S8_ONOFF = on_off;
		for (i = 0; i < 8; i++) {
			S_power[i] = on_off;	//uloen stavu o zap/vyp vech serv
		}
		break;
	}
}
//****************************************************************************************************
//****************************** ZATEK HLAVNHO PROGRAMU *******************************************
//****************************************************************************************************
void main(void) {
	/* Write your local variable definition here */
	/*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
	PE_low_level_init();
	/*** End of Processor Expert internal initialization.                    ***/

	/* Write your code here */
	/* For example: for(;;) { } */
	S_move_flag = 0xFF;	//natoen serv do vchoz polohy

	//============================== ZATEK NEKONEN SMYKY ==================================================
	while (1) {
		SRS = 0x55;				//resetovani WatchDogTimeru pro hard reset 
		SRS = 0xAA;
		//====================================================================================================
		//========== PRICHOZI ZPRAVA PO CANU ==========
		if (RX_data_ready == 1) {
			RX_data_ready = 0;
			LED_RGB_blink = 1;							//bliknut RGB
			RGB_color(BLUE);							//modr barva
			//========== MANULN OVLDN SERV ==========
			if (RX_message[4] < 0xB6) {
				for (i = 4; i < 12; i++) {
					if (RX_message[i] < 0xB5) {
						if (S_power[i - 4] == 1) {//pokud je pslun servo vypnut
							S_ON_OFF(i - 3, 0);			//zapni ho
						}
						S_pos[i - 4] = (RX_message[i] * 0x05) + 0x104; //vypoet dorazu od 0x00 do 0xB4 (0 - 180)
						S_resp[i - 4] =
								(uint8_t) ((S_pos[i - 4] - 0x104) / 0x05); //zptn vazba o natoen serv
						S_move_flag = 0xFF;
						if ((error[1] & 0x20) == 0x20) { //nastala pedtm chyba?
							error[1] = error[1] ^ 0x20;		//vynulovn chyby
						}
					} else if (RX_message[i] == 0xB5) {
						S_ON_OFF(i - 3, 1);			//vypnut pslunho serva
						S_resp[i - 4] = 0xB5;		//informace o vypnut serva
						if ((error[1] & 0x20) == 0x20) {//nastala pedtm chyba?
							error[1] = error[1] ^ 0x20;		//vynulovn chyby
						}
					} else {
						error[1] = error[1] | 0x20;			//chyba
						send_error = 1;					//povolen zasln chyby
					}
				}
			}
			switch (RX_message[4]) { //precti zpravu na ctvrte pozici			
			//---------- HARD RESET JEDNOTKY ----------
			case 0xB6:
				if (RX_message[5] == 0x01) {
					SRS = 0x00;			//hard reset
				} else {
					error[1] = error[1] | 0x80;			//chyba
					send_error = 1;						//povolen zasln chyby
				}
				break;
				//---------- NASTAVEN ASU MEZI OTENM JEDNOTLIVCH SERV ----------
			case 0xB7:
				if ((RX_message[5] >= 0x0A) && (RX_message[5] <= 0xFA)) {//prodleva 100 a 2500 ms
					S_move_delay = RX_message[5];
					if ((error[1] & 0x08) == 0x08) {	//nastala pedtm chyba?
						error[1] = error[1] ^ 0x08;		//vynulovn chyby
					}
				} else {
					error[1] = error[1] | 0x08;		//chyba
					send_error = 1;					//povolen zasln chyby
				}
				break;
				//---------- ZPTN ZPRVA O AKTULNM STAVU NATOEN SERV ----------
			case 0xB8:
				if (RX_message[5] == 0x01) {//pozice serv vlevo, vpravo, mezi
					send_CAN = CAN_TX(CAN_TX_ID_1, ID_stand, &S_resp_LR, 8);
					if ((error[1] & 0x04) == 0x04) {	//nastala pedtm chyba?
						error[1] = error[1] ^ 0x04;		//vynulovn chyby
					}
				} else if (RX_message[5] == 0x02) {	//pozice serv ve stupnch
					send_CAN = CAN_TX(CAN_TX_ID_1, ID_stand, &S_resp, 8);
					if ((error[1] & 0x04) == 0x04) {	//nastala pedtm chyba?
						error[1] = error[1] ^ 0x04;		//vynulovn chyby
					}
				} else {
					error[1] = error[1] | 0x04;		//chyba
					send_error = 1;					//povolen zasln chyby
				}
				break;
				//---------- NASTAVEN SOFTWAROVCH DORAZ SERV ----------
			case 0xB9:
				if (((RX_message[5] >= 0x00) && (RX_message[5] <= 0x07))
						|| (RX_message[5] == 0xAA)) {
					if ((error[1] & 0x10) == 0x10) {	//nastala pedtm chyba?
						error[1] = error[1] ^ 0x10;		//vynulovn chyby
					}
					switch (RX_message[5]) {
					case 0x00:	//servo1
						S_stopL[0] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[0] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0x01:	//servo2
						S_stopL[1] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[1] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0x02:	//servo3
						S_stopL[2] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[2] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0x03:	//servo4
						S_stopL[3] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[3] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0x04:	//servo5
						S_stopL[4] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[4] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0x05:	//servo6
						S_stopL[5] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[5] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0x06:	//servo7
						S_stopL[6] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[6] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0x07:	//servo8
						S_stopL[7] = (RX_message[6] * 0x05) + 0x104;//lev doraz
						S_stopR[7] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						break;
					case 0xAA:	//vechny serva
						for (i = 0; i < 8; i++) {
							S_stopL[i] = (RX_message[6] * 0x05) + 0x104;//lev doraz
							S_stopR[i] = (RX_message[7] * 0x05) + 0x104;//prav doraz
						}
						break;
					}
				} else {
					error[1] = error[1] | 0x10;		//chyba
					send_error = 1;					//povolen zasln chyby
				}
				break;
				//---------- PESTAVEN DOLEVA/DOPRAVA ----------
			case 0xBA:
				//1 = doleva, 0 = doprava
				//----- servo 1 -----
				if ((RX_message[5] & 0x01) == 0x01) {//pokud je zvoleno servo 1
					if ((RX_message[6] & 0x01) == 0x01) {	//pehozen doleva?
						S_pos[0] = S_stopL[0];				//doleva
						S_move_flag = S_move_flag | 0x01; //povolen pohyb serva
						S_resp_LR[0] = 0xF1;	//zptn vazba
					} else {
						S_pos[0] = S_stopR[0];				//doprava
						S_move_flag = S_move_flag | 0x01; //povolen pohyb serva
						S_resp_LR[0] = 0xF0;	//zptn vazba
					}
					S_resp[0] = (uint8_t) ((S_pos[0] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[0] = 0xF3;	//informace o vypnut serva
				}
				//----- servo 2 -----
				if ((RX_message[5] & 0x02) == 0x02) {//pokud je zvoleno servo 2
					if ((RX_message[6] & 0x02) == 0x02) {	//pehozen doleva?
						S_pos[1] = S_stopL[1];				//doleva
						S_move_flag = S_move_flag | 0x02; //povolen pohyb serva
						S_resp_LR[1] = 0xF1;	//zptn vazba
					} else {
						S_pos[1] = S_stopR[1];				//doprava
						S_move_flag = S_move_flag | 0x02; //povolen pohyb serva
						S_resp_LR[1] = 0xF0;	//zptn vazba
					}
					S_resp[1] = (uint8_t) ((S_pos[1] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[1] = 0xF3;	//informace o vypnut serva
				}
				//----- servo 3 -----
				if ((RX_message[5] & 0x04) == 0x04) {//pokud je zvoleno servo 3
					if ((RX_message[6] & 0x04) == 0x04) {	//pehozen doleva?
						S_pos[2] = S_stopL[2];	//doleva
						S_move_flag = S_move_flag | 0x04; //povolen pohyb serva
						S_resp_LR[2] = 0xF1;	//zptn vazba
					} else {
						S_pos[2] = S_stopR[2];	//doprava
						S_move_flag = S_move_flag | 0x04; //povolen pohyb serva
						S_resp_LR[2] = 0xF0;	//zptn vazba
					}
					S_resp[2] = (uint8_t) ((S_pos[2] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[2] = 0xF3;	//informace o vypnut serva
				}
				//----- servo 4 -----
				if ((RX_message[5] & 0x08) == 0x08) {//pokud je zvoleno servo 4
					if ((RX_message[6] & 0x08) == 0x08) {	//pehozen doleva?
						S_pos[3] = S_stopL[3];	//doleva
						S_move_flag = S_move_flag | 0x08; //povolen pohyb serva
						S_resp_LR[3] = 0xF1;	//zptn vazba
					} else {
						S_pos[3] = S_stopR[3];	//doprava
						S_move_flag = S_move_flag | 0x08; //povolen pohyb serva
						S_resp_LR[3] = 0xF0;	//zptn vazba
					}
					S_resp[3] = (uint8_t) ((S_pos[3] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[3] = 0xF3;	//informace o vypnut serva
				}
				//----- servo 5 -----
				if ((RX_message[5] & 0x10) == 0x10) {//pokud je zvoleno servo 5
					if ((RX_message[6] & 0x10) == 0x10) {	//pehozen doleva?
						S_pos[4] = S_stopL[4];	//doleva
						S_move_flag = S_move_flag | 0x10; //povolen pohyb serva
						S_resp_LR[4] = 0xF1;	//zptn vazba
					} else {
						S_pos[4] = S_stopR[4];	//doprava
						S_move_flag = S_move_flag | 0x10; //povolen pohyb serva
						S_resp_LR[4] = 0xF0;	//zptn vazba
					}
					S_resp[4] = (uint8_t) ((S_pos[4] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[4] = 0xF3;	//informace o vypnut serva
				}
				//----- servo 6 -----
				if ((RX_message[5] & 0x20) == 0x20) {//pokud je zvoleno servo 6
					if ((RX_message[6] & 0x20) == 0x20) {	//pehozen doleva?
						S_pos[5] = S_stopL[5];	//doleva
						S_move_flag = S_move_flag | 0x20; //povolen pohyb serva
						S_resp_LR[5] = 0xF1;	//zptn vazba
					} else {
						S_pos[5] = S_stopR[5];	//doprava
						S_move_flag = S_move_flag | 0x20; //povolen pohyb serva
						S_resp_LR[5] = 0xF0;	//zptn vazba
					}
					S_resp[5] = (uint8_t) ((S_pos[5] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[5] = 0xF3;	//informace o vypnut serva
				}
				//----- servo 7 -----
				if ((RX_message[5] & 0x40) == 0x40) {//pokud je zvoleno servo 7
					if ((RX_message[6] & 0x40) == 0x40) {	//pehozen doleva?
						S_pos[6] = S_stopL[6];	//doleva
						S_move_flag = S_move_flag | 0x40; //povolen pohyb serva
						S_resp_LR[6] = 0xF1;	//zptn vazba
					} else {
						S_pos[6] = S_stopR[6];	//doprava
						S_move_flag = S_move_flag | 0x40; //povolen pohyb serva
						S_resp_LR[6] = 0xF0;	//zptn vazba
					}
					S_resp[6] = (uint8_t) ((S_pos[6] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[6] = 0xF3;	//informace o vypnut serva
				}
				//----- servo 8 -----
				if ((RX_message[5] & 0x80) == 0x80) {//pokud je zvoleno servo 8
					if ((RX_message[5] & 0x80) == 0x80) {	//pehozen doleva?
						S_pos[7] = S_stopL[7];	//doleva
						S_move_flag = S_move_flag | 0x80; //povolen pohyb serva
						S_resp_LR[7] = 0xF1;	//zptn vazba
					} else {
						S_pos[7] = S_stopR[7];	//doprava
						S_move_flag = S_move_flag | 0x80; //povolen pohyb serva
						S_resp_LR[7] = 0xF0;	//zptn vazba
					}
					S_resp[7] = (uint8_t) ((S_pos[7] - 0x104) / 0x05); //zptn vazba ve stupnch
				} else {
					S_resp_LR[7] = 0xF3;	//informace o vypnut serva
				}
				send_LR = 1;//povolen zasln zprvy o pozici serv vlevo/vpravo
				break;
				//---------- NASTAVEN ID SLA JEDNOTKY ----------
			case 0xBB:
				//TODO
				break;
				//--------------------------------------------------
			default:
				break;
			}
		}
		//====================================================================================================
		//========== PESTAVEN JEDNOTLIVCH VHYBEK ==========
		if (!prodleva && S_move_flag) {
			S_move_inc++;					//pro vbr serva
			prodleva = S_move_delay;//zpodn mezi pohybem jednotlivch serv
		}
		switch (S_move_inc) {
		//----- servo 1 -----
		case 1:
			if ((S_move_flag & 0x01) == 0x01) {	//m se pestavit tato vhybka?
				if ((S_pos[0] >= S_stopL[0]) && (S_pos[0] <= S_stopR[0])) { //hldn doraz
					S_ON_OFF(1, 0); 	//zapnut serva pro pestaven
					S1_PWM = S_pos[0];			//pestaven
				} else {
					S_ON_OFF(1, 1); 			//vypnut serva
					S_resp_LR[0] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
			//----- servo 2 -----
		case 2:
			S_ON_OFF(1, 1); //vypnut pedchozho serva po pestaven
			if ((S_move_flag & 0x02) == 0x02) {	//m se pestavit tato vhybka?
				if ((S_pos[1] >= S_stopL[1]) && (S_pos[1] <= S_stopR[1])) { //hldn doraz
					S_ON_OFF(2, 0); 	//zapnut serva pro pestaven
					S2_PWM = S_pos[1];			//pestaven
				} else {
					S_ON_OFF(2, 1); 			//vypnut serva
					S_resp_LR[1] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
			//----- servo 3 -----
		case 3:
			S_ON_OFF(2, 1); //vypnut pedchozho serva po pestaven
			if ((S_move_flag & 0x04) == 0x04) {	//m se pestavit tato vhybka?
				if ((S_pos[2] >= S_stopL[2]) && (S_pos[2] <= S_stopR[2])) { //hldn doraz
					S_ON_OFF(3, 0); 	//zapnut serva pro pestaven
					S3_PWM = S_pos[2];			//pestaven
				} else {
					S_ON_OFF(3, 1); 			//vypnut serva
					S_resp_LR[2] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
			//----- servo 4 -----
		case 4:
			S_ON_OFF(3, 1); //vypnut pedchozho serva po pestaven
			if ((S_move_flag & 0x08) == 0x08) {	//m se pestavit tato vhybka?
				if ((S_pos[3] >= S_stopL[3]) && (S_pos[3] <= S_stopR[3])) { //hldn doraz
					S_ON_OFF(4, 0); 	//zapnut serva pro pestaven
					S4_PWM = S_pos[3];			//pestaven
				} else {
					S_ON_OFF(4, 1); 			//vypnut serva
					S_resp_LR[3] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
			//----- servo 5 -----
		case 5:
			S_ON_OFF(4, 1); //vypnut pedchozho serva po pestaven
			if ((S_move_flag & 0x10) == 0x10) {	//m se pestavit tato vhybka?
				if ((S_pos[4] >= S_stopL[4]) && (S_pos[4] <= S_stopR[4])) { //hldn doraz
					S_ON_OFF(5, 0); 	//zapnut serva pro pestaven
					S5_PWM = S_pos[4];			//pestaven
				} else {
					S_ON_OFF(5, 1); 			//vypnut serva
					S_resp_LR[4] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
			//----- servo 6 -----
		case 6:
			S_ON_OFF(5, 1); //vypnut pedchozho serva po pestaven
			if ((S_move_flag & 0x20) == 0x20) {	//m se pestavit tato vhybka?
				if ((S_pos[5] >= S_stopL[5]) && (S_pos[5] <= S_stopR[5])) { //hldn doraz
					S_ON_OFF(6, 0); 	//zapnut serva pro pestaven
					S6_PWM = S_pos[5];			//pestaven
				} else {
					S_ON_OFF(6, 1); 			//vypnut serva
					S_resp_LR[5] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
			//----- servo 7 -----
		case 7:
			S_ON_OFF(6, 1); //vypnut pedchozho serva po pestaven
			if ((S_move_flag & 0x40) == 0x40) {	//m se pestavit tato vhybka?
				if ((S_pos[6] >= S_stopL[6]) && (S_pos[6] <= S_stopR[6])) { //hldn doraz
					S_ON_OFF(7, 0); 	//zapnut serva pro pestaven
					S7_PWM = S_pos[6];			//pestaven
				} else {
					S_ON_OFF(7, 1); 			//vypnut serva
					S_resp_LR[6] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
			//----- servo 8 -----
		case 8:
			S_ON_OFF(7, 1); //vypnut pedchozho serva po pestaven
			if ((S_move_flag & 0x80) == 0x80) {	//m se pestavit tato vhybka?
				if ((S_pos[7] >= S_stopL[7]) && (S_pos[7] <= S_stopR[7])) { //hldn doraz
					S_ON_OFF(8, 0); 	//zapnut serva pro pestaven
					S8_PWM = S_pos[7];			//pestaven
				} else {
					S_ON_OFF(8, 1); 			//vypnut serva
					S_resp_LR[7] = 0xF3;	//informace o vypnut serva
				}
			}
			break;
		case 9:
			S_ON_OFF(8, 1); //vypnut pedchozho serva po pestaven
			//S_ON_OFF(10, 1);	//vypnut vech serv
			S_move_inc = 0;
			S_move_flag = 0x00;	//natoen serv se provede jen jednou
			send_once = 1;		//odele jednou zprvu
			break;
		default:
			S_move_inc = 0;
			S_move_flag = 0x00;
			break;
		}
		//====================================================================================================
		//========== ODESILANI PO CANU ==========
		if (send_once && !send_LR) {	//odele pozici serv ve stupnch
			send_CAN = CAN_TX(CAN_TX_ID_1, ID_stand, &S_resp, 8); //odesln pozic serv
			send_once = 0;
		}
		if (send_once && send_LR) {	//odele pozici serv vlevo/vpravo/uprosted
			send_CAN = CAN_TX(CAN_TX_ID_1, ID_stand, &S_resp_LR, 8);//odesln pozic serv
			send_LR = 0;
		}
		//====================================================================================================
		//========== SIGNALIZACE A HLASENI CHYBY ==========
		if (error[1] != 0x00) {						//pokud nkde nastala chyba
			if ((error[1] & 0x40) == 0x40) {	//chyba pi pestavovn vhybek
				RGB_color(RED);							//erven barva
			}
			if ((error[1] & 0x20) == 0x20) { 	//chyba pi pesnm polohovn
				RGB_color(YELLOW);						//lut barva
			}
			if ((error[1] & 0x10) == 0x10) { //chyba pi nastavovn sw doraz
				RGB_color(AZURE);						//tyrkysov barva
			}
			LED_RGB_R = R;
			LED_RGB_G = G;
			LED_RGB_B = B;
			if (send_error == 1) {				//pokud m poslat zprvu o chyb
				send_CAN = CAN_TX(CAN_TX_ID_1, ID_stand, &error, 3);//hlen chyby
				send_error = 0;					//vynulovn pro dal odesln
			}
		}
		//====================================================================================================
		//========== BLIKANI LED ==========
		if (LED_RGB_blink > 0) {		//bliknut LED?
			LED_RGB_timer = 20;		//blikej periodou 20
			LED_RGB_R = R;
			LED_RGB_G = G;
			LED_RGB_B = B;
			LED_RGB_blink = 0;
		}
		if (LED_RGB_timer == 0) {	//zhasnut RGB
			LED_RGB_R = 1;
			LED_RGB_G = 1;
			LED_RGB_B = 1;
		}
		//====================================================================================================
	}
//============================== KONEC NEKONEN SMYKY ==================================================

//****************************************************************************************************
//****************************** KONEC HLAVNIHO PROGRAMU *********************************************
//****************************************************************************************************
	/*** Don't write any code pass this line, or it will be deleted during code generation. ***/
	/*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
#ifdef PEX_RTOS_START
	PEX_RTOS_START(); /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
#endif
	/*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;){}
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
 ** @}
 */
/*
 ** ###################################################################
 **
 **     This file was created by Processor Expert 10.3 [05.09]
 **     for the Freescale HCS08 series of microcontrollers.
 **
 ** ###################################################################
 */
